@@ -1,21 +1,21 @@
---[[
+--[[ $%BEGINLICENSE%$
+ Copyright (C) 2007-2008 MySQL AB, 2008 Sun Microsystems, Inc
 
-   Copyright (C) 2007 MySQL AB
+ This program is free software; you can redistribute it and/or modify
+ it under the terms of the GNU General Public License as published by
+ the Free Software Foundation; version 2 of the License.
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; version 2 of the License.
+ This program is distributed in the hope that it will be useful,
+ but WITHOUT ANY WARRANTY; without even the implied warranty of
+ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ GNU General Public License for more details.
 
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+ You should have received a copy of the GNU General Public License
+ along with this program; if not, write to the Free Software
+ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
-   You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ $%ENDLICENSE%$ --]]
 
---]]
 
 ---
 --[[
@@ -44,6 +44,8 @@ DEBUG = DEBUG + 0
 
 local xtab_version = '0.1.3'
 
+local tokenizer = require("proxy.tokenizer")
+
 --[[
    error status for the xtab sequence
    if an error happens in a query before the last one,
@@ -178,7 +180,7 @@ function read_query( packet )
     -- simple tokeninzing the query, looking for accepted pattern
     --
     local option, table_name, row_field, col_field , op, op_col , summary
-    local query_tokens = proxy.tokenize(query)
+    local query_tokens = tokenizer.tokenize(query)
     local START_TOKEN = 0
 
     if  ( query_tokens[1]['text']:lower() == 'xtab' )
@@ -280,7 +282,8 @@ function read_query( packet )
     --]]
     proxy.queries:append(xtab_id_before, 
         string.char(proxy.COM_QUERY) .. 
-        "set group_concat_max_len = 1024*1024" )
+        "set group_concat_max_len = 1024*1024",
+	{ resultset_is_needed = true })
 
     --[[
        If further queries need to be executed before the 
@@ -309,7 +312,8 @@ function read_query( packet )
             col_field, 
             table_name, 
             col_field
-        )
+        ),
+	{ resultset_is_needed = true }
     )
     return proxy.PROXY_SEND_QUERY
 end
@@ -393,7 +397,8 @@ function read_query_result(inj)
         --
         -- The XTAB query is executed
         --
-        proxy.queries:append(xtab_id_exec, string.char(proxy.COM_QUERY) .. xtab_query)
+        proxy.queries:append(xtab_id_exec, string.char(proxy.COM_QUERY) .. xtab_query,
+            { resultset_is_needed = true })
         print_debug (xtab_query, 2)
         return proxy.PROXY_IGNORE_RESULT
     end 
