@@ -626,12 +626,13 @@ int main_cmdline(int argc, char **argv) {
 		 * macosx has .so or .dylib
 		 * hpux has .sl
 		 */ 
-		gchar *path = g_build_filename(base_dir, "lib", "mysql-proxy", "lua", "?." G_MODULE_SUFFIX, NULL);
 #  if _WIN32
+		gchar *path = g_build_filename(base_dir, "bin", "lua-?." G_MODULE_SUFFIX, NULL);
 		gchar *env_path = g_strdup_printf("%s=%s", LUA_CPATH, path);
 		_putenv(env_path);
 		g_free(env_path);
 #  else
+		gchar *path = g_build_filename(base_dir, "lib", "mysql-proxy", "lua", "?." G_MODULE_SUFFIX, NULL);
 		g_setenv(LUA_CPATH, path, 1);
 #  endif
 		if (print_version) printf("    LUA_CPATH: %s" CHASSIS_NEWLINE, path);
@@ -669,7 +670,6 @@ int main_cmdline(int argc, char **argv) {
 	
 	/* Lets find the plugin directory relative the executable path */
 	if (!plugin_dir) {
-		/* for Win32 the default plugin dir is bin/ and not lib/package_name/plugins */
 #ifdef WIN32
 		plugin_dir = g_build_filename(srv->base_dir, "bin", NULL);
 #else
@@ -722,11 +722,12 @@ int main_cmdline(int argc, char **argv) {
 		((strlen(executable_name) >= sizeof(pname) - 1) && \
 		0 == strcmp(executable_name + strlen(executable_name) - (sizeof(pname) - 1), pname) \
 		)
-
-		/* on Windows allow for the executable suffix */
 #ifdef WIN32
-		if(g_str_has_suffix(argv[0], ".exe")) {
-			executable_name = g_strndup(argv[0], strlen(argv[0]) - 4);	/* 4 is the length of ".exe" */
+		/* on Windows allow for the executable and -svc suffix */
+		if (g_str_has_suffix(argv[0], "-svc.exe")) {
+			executable_name = g_strndup(argv[0], strlen(argv[0]) - (sizeof("-svc.exe") - 1));
+		} else if (g_str_has_suffix(argv[0], ".exe")) {
+			executable_name = g_strndup(argv[0], strlen(argv[0]) - (sizeof(".exe") - 1));
 		} else {
 			executable_name = g_strdup(argv[0]);
 		}
@@ -753,7 +754,7 @@ int main_cmdline(int argc, char **argv) {
 	/* load the plugins */
 	for (i = 0; plugin_names && plugin_names[i]; i++) {
 #ifdef WIN32
-#define G_MODULE_PREFIX ""
+#define G_MODULE_PREFIX "plugin-" /* we build the plugins with a prefix on win32 to avoid name-clashing in bin/ */
 #else
 #define G_MODULE_PREFIX "lib"
 #endif
@@ -1150,7 +1151,7 @@ int main_win32(int argc, char **argv) {
  * We eventually fall down through to main_cmdline, even on Windows.
  */
 int main(int argc, char **argv) {
-#ifdef WIN32
+#ifdef WIN32_AS_SERVICE
 	return main_win32(argc, argv);
 #else
 	return main_cmdline(argc, argv);
