@@ -1,21 +1,26 @@
-/* Copyright (C) 2007 MySQL AB
+/* $%BEGINLICENSE%$
+ Copyright (C) 2007-2008 MySQL AB, 2008 Sun Microsystems, Inc
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; version 2 of the License.
+ This program is free software; you can redistribute it and/or modify
+ it under the terms of the GNU General Public License as published by
+ the Free Software Foundation; version 2 of the License.
 
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+ This program is distributed in the hope that it will be useful,
+ but WITHOUT ANY WARRANTY; without even the implied warranty of
+ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ GNU General Public License for more details.
 
-   You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */ 
+ You should have received a copy of the GNU General Public License
+ along with this program; if not, write to the Free Software
+ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+ $%ENDLICENSE%$ */
+ 
 
 #include <glib.h>
 
 #include "network-conn-pool.h"
+#include "network-mysqld-packet.h"
 #include "glib-ext.h"
 #include "sys-pedantic.h"
 
@@ -34,7 +39,7 @@
  *
  * @return a connection pool entry
  */
-network_connection_pool_entry *network_connection_pool_entry_init(void) {
+network_connection_pool_entry *network_connection_pool_entry_new(void) {
 	network_connection_pool_entry *e;
 
 	e = g_new0(network_connection_pool_entry, 1);
@@ -68,7 +73,7 @@ void network_connection_pool_entry_free(network_connection_pool_entry *e, gboole
  *
  * @param q a GQueue to free
  *
- * @see network_connection_pool_init
+ * @see network_connection_pool_new
  * @see GDestroyFunc
  */
 static void g_queue_free_all(gpointer q) {
@@ -80,10 +85,13 @@ static void g_queue_free_all(gpointer q) {
 	g_queue_free(queue);
 }
 
+network_connection_pool *network_connection_pool_init(void) {
+	return network_connection_pool_new();
+}
 /**
  * init a connection pool
  */
-network_connection_pool *network_connection_pool_init(void) {
+network_connection_pool *network_connection_pool_new(void) {
 	network_connection_pool *pool;
 
 	pool = g_new0(network_connection_pool, 1);
@@ -209,7 +217,7 @@ network_connection_pool_entry *network_connection_pool_add(network_connection_po
 	network_connection_pool_entry *entry;
 	GQueue *conns = NULL;
 
-	entry = network_connection_pool_entry_init();
+	entry = network_connection_pool_entry_new();
 	entry->sock = sock;
 	entry->pool = pool;
 
@@ -219,10 +227,10 @@ network_connection_pool_entry *network_connection_pool_add(network_connection_po
 	g_debug("%s: (add) adding socket to pool for user '%s' -> %p", G_STRLOC, sock->username->str, sock);
 #endif
 
-	if (NULL == (conns = g_hash_table_lookup(pool->users, sock->username))) {
+	if (NULL == (conns = g_hash_table_lookup(pool->users, sock->response->username))) {
 		conns = g_queue_new();
 
-		g_hash_table_insert(pool->users, g_string_dup(sock->username), conns);
+		g_hash_table_insert(pool->users, g_string_dup(sock->response->username), conns);
 	}
 
 	g_queue_push_tail(conns, entry);
@@ -237,7 +245,7 @@ void network_connection_pool_remove(network_connection_pool *pool, network_conne
 	network_socket *sock = entry->sock;
 	GQueue *conns;
 
-	if (NULL == (conns = g_hash_table_lookup(pool->users, sock->username))) {
+	if (NULL == (conns = g_hash_table_lookup(pool->users, sock->response->username))) {
 		return;
 	}
 
