@@ -1,3 +1,20 @@
+--[[ $%BEGINLICENSE%$
+ Copyright (C) 2007-2008 MySQL AB, 2008 Sun Microsystems, Inc
+
+ This program is free software; you can redistribute it and/or modify
+ it under the terms of the GNU General Public License as published by
+ the Free Software Foundation; version 2 of the License.
+
+ This program is distributed in the hope that it will be useful,
+ but WITHOUT ANY WARRANTY; without even the implied warranty of
+ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ GNU General Public License for more details.
+
+ You should have received a copy of the GNU General Public License
+ along with this program; if not, write to the Free Software
+ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+ $%ENDLICENSE%$ --]]
 
 -- vim:sw=4:noexpandtab 
 
@@ -9,6 +26,7 @@
 
 -- we require LFS (LuaFileSystem)
 require("lfs")
+require("glib2")
 
 ---
 -- get the directory-name of a path
@@ -27,7 +45,7 @@ function dirname(filename)
 	
 	attr = assert(lfs.attributes(dirname))
 
-	assert(attr.mode == "directory")
+	assert(attr.mode == "directory", "dirname("..filename..") failed: is ".. attr.mode)
 
 	return dirname
 end
@@ -73,18 +91,20 @@ PROXY_PORT		  	= os.getenv("PROXY_PORT")		   or "14040"
 PROXY_MASTER_PORT   = os.getenv("PROXY_MASTER_PORT")	or "14050"
 PROXY_SLAVE_PORT	= os.getenv("PROXY_SLAVE_PORT")	 	or "14060"
 PROXY_CHAIN_PORT	= os.getenv("PROXY_CHAIN_PORT")	 	or "14070"
-ADMIN_PORT		  	= os.getenv("ADMIN_PORT")		   	or "14041"
-ADMIN_MASTER_PORT   = os.getenv("ADMIN_MASTER_PORT")	or "14051"
-ADMIN_SLAVE_PORT	= os.getenv("ADMIN_SLAVE_PORT")	 	or "14061"
-ADMIN_CHAIN_PORT	= os.getenv("ADMIN_CHAIN_PORT")	 	or "14071"
+ADMIN_PORT		  	= os.getenv("ADMIN_PORT")		   	or "14045"
+ADMIN_MASTER_PORT   = os.getenv("ADMIN_MASTER_PORT")	or "14055"
+ADMIN_SLAVE_PORT	= os.getenv("ADMIN_SLAVE_PORT")	 	or "14065"
+ADMIN_CHAIN_PORT	= os.getenv("ADMIN_CHAIN_PORT")	 	or "14075"
 -- local PROXY_TMP_LUASCRIPT = os.getenv("PROXY_TMP_LUASCRIPT") or "/tmp/proxy.tmp.lua"
 
 local srcdir		 = os.getenv("srcdir")		 	or testdir .. "/"
-local builddir	   = os.getenv("builddir")	   		or testdir .. "/../"
+local top_builddir	 = os.getenv("top_builddir")   		or testdir .. "/../"
+local builddir	         = os.getenv("builddir")	   	or testdir .. "/" -- same as srcdir by default
 
 local PROXY_TRACE	= os.getenv("PROXY_TRACE")		or ""	-- use it to inject strace or valgrind
 local PROXY_PARAMS   = os.getenv("PROXY_PARAMS")   	or ""	-- extra params
-local PROXY_BINPATH  = os.getenv("PROXY_BINPATH")  	or builddir .. "/src/mysql-proxy"
+local PROXY_BINPATH  = os.getenv("PROXY_BINPATH")  	or top_builddir .. "/src/mysql-proxy"
+PROXY_LIBPATH  = os.getenv("PROXY_LIBPATH")  	or top_builddir .. "/plugins/"
 
 local COVERAGE_LCOV  = os.getenv("COVERAGE_LCOV")
 
@@ -97,6 +117,8 @@ PROXY_MASTER_PIDFILE  	= lfs.currentdir() .. "/mysql-proxy-test-master.pid"
 PROXY_SLAVE_PIDFILE   	= lfs.currentdir() .. "/mysql-proxy-test-slave.pid"
 PROXY_CHAIN_PIDFILE   	= lfs.currentdir() .. "/mysql-proxy-test-chain.pid"
 PROXY_BACKEND_PIDFILE 	= lfs.currentdir() .. "/mysql-proxy-test-backend.pid"
+PROXY_TEST_BASEDIR		= lfs.currentdir()
+
 DEFAULT_SCRIPT_FILENAME	 = "/tmp/dummy.lua"
 default_proxy_options = {
 	["proxy-backend-addresses"] = MYSQL_HOST .. ":" .. MYSQL_PORT,
@@ -104,6 +126,8 @@ default_proxy_options = {
 	["admin-address"]		   	= PROXY_HOST .. ":" .. ADMIN_PORT,
 	["pid-file"]				= PROXY_PIDFILE,
 	["proxy-lua-script"]		= DEFAULT_SCRIPT_FILENAME,
+	["plugin-dir"]			= PROXY_LIBPATH,
+	["basedir"]					= PROXY_TEST_BASEDIR,
 	}
 
 default_master_options = {
@@ -112,6 +136,8 @@ default_master_options = {
 	["admin-address"]		   	= PROXY_HOST .. ":" .. ADMIN_MASTER_PORT,
 	["pid-file"]				= PROXY_MASTER_PIDFILE,
 	["proxy-lua-script"]		= DEFAULT_SCRIPT_FILENAME,
+	["plugin-dir"]			= PROXY_LIBPATH,
+	["basedir"]					= PROXY_TEST_BASEDIR,
 	}
 
 default_slave_options = {
@@ -120,8 +146,11 @@ default_slave_options = {
 	["admin-address"]		  	= PROXY_HOST .. ":" .. ADMIN_SLAVE_PORT,
 	["pid-file"]				= PROXY_SLAVE_PIDFILE,
 	["proxy-lua-script"]		= DEFAULT_SCRIPT_FILENAME,
+	["plugin-dir"]			= PROXY_LIBPATH,
+	["basedir"]					= PROXY_TEST_BASEDIR,
 	}
 
+
 tests_to_skip = {}
 local tests_to_skip_filename = 'tests_to_skip.lua' 
 local proxy_list = {}
@@ -205,8 +234,18 @@ function options_tostring(tbl, sep)
 
 	local s = ""
 	for k, v in pairs(tbl) do
-		local enc_value = v:gsub("\\", "\\\\"):gsub("\"", "\\\"")
-		s = s .. "--" .. k .. "=\"" .. enc_value .. "\" "
+		local values
+		-- if the value is a table, repeat the option
+		if type(v) == "table" then
+			values = v
+		else
+			values = { v }
+		end
+
+		for tk, tv in pairs(values) do
+			local enc_value = tv:gsub("\\", "\\\\"):gsub("\"", "\\\"")
+			s = s .. "--" .. k .. "=\"" .. enc_value .. "\" "
+		end
 	end
 	-- print_verbose(" option: " .. s)
 
@@ -234,8 +273,7 @@ function os_execute(cmdline)
 end
 
 function get_pid(pid_file_name)
-	local fh = assert(io.open(pid_file_name, 'r'),
-		"error opening " .. pid_file_name)
+	local fh = assert(io.open(pid_file_name, 'r'))
 	local pid = assert(fh:read() ,
 		"PID not found in " .. pid_file_name)
 	fh:close()
@@ -244,23 +282,45 @@ end
 
 function wait_proc_up(pid_file) 
 	local rounds = 0
-	os.execute("sleep 1") -- wait until the pid-file is created
+
+	while not file_exists(pid_file) do
+		glib2.usleep(200 * 1000) -- wait until the pid-file is created
+
+		rounds = rounds + 1
+		print_verbose(("(wait_proc_up) pid-wait: %d rounds, (%s)"):format(rounds, pid_file))
+
+		if rounds > 1000 then error(("proxy failed to start: no pid-file %s"):format(pid_file)) end
+	end
 
 	local pid = get_pid(pid_file)
 
+	rounds = 0
+	-- check that the process referenced in the PID-file is still up
 	while 0 ~= os.execute("kill -0 ".. pid .."  2> /dev/null") do
-		os.execute("sleep 1")
+		glib2.usleep(200 * 1000) -- wait until the pid-file is created
 		rounds = rounds + 1
 		print_verbose(("(wait_proc_up) kill-wait: %d rounds, pid=%d (%s)"):format(rounds, pid, pid_file))
+
+		if rounds > 5 then error(("proxy seems to have crashed: pid=%d (%s)"):format(pid, pid_file)) end
 	end
 end
 
+function proc_is_up(pid)
+	return os.execute("kill -0 ".. pid .."  2> /dev/null")
+end
+
+function proc_stop(pid)
+	return os.execute("kill -TERM ".. pid)
+end
+
 function wait_proc_down(pid_file) 
 	local rounds = 0
 	local pid = get_pid(pid_file)
 
-	while 0 == os.execute("kill -0 ".. pid .."  2> /dev/null") do
-		os.execute("sleep 1")
+	-- wait until the proc in the pid file is dead
+	-- the shutdown takes at about 500ms
+	while 0 == proc_is_up(pid) do
+		glib2.usleep(200 * 1000) -- wait until process is gone
 		rounds = rounds + 1
 		print_verbose(("(wait_proc_down) kill-wait: %d rounds, pid=%d (%s)"):format(rounds, pid, pid_file))
 	end
@@ -270,19 +330,34 @@ function stop_proxy()
 	-- shut dowm the proxy
 	--
 	-- win32 has tasklist and taskkill on the shell
-	-- 
+	 
+	
 	-- shuts down every proxy in the proxy list
 	--
 	for proxy_name, proxy_options in pairs(proxy_list) do
+		local pid
 		pid_file = proxy_options['pid-file']
 		print_verbose ('stopping proxy ' .. proxy_name)
-		if 0 == os.execute("kill -TERM  ".. get_pid(pid_file) ) then
-			wait_proc_down(pid_file)
+		
+		pid = get_pid(pid_file)
+
+		if proc_is_up(pid) then
+			proc_stop(pid)
 		else
-			-- hmm, if it failed ... not good, perhaps it already crashed
+			print("-- process "..proxy_name.." is already down")
+			exitcode = -1
 		end
+	end
+
+	-- wait until they are all gone
+	for proxy_name, proxy_options in pairs(proxy_list) do
+		pid_file = proxy_options['pid-file']
+
+		wait_proc_down(pid_file)
+
 		os.remove(pid_file)
 	end
+
 	--
 	-- empties the proxy list
 	--
@@ -361,6 +436,8 @@ function after_test()
 end
 
 function alternative_execute (cmd)
+	print_verbose(cmd)
+
 	local fh = io.popen(cmd)
 	assert(fh, 'error executing '.. cmd)
 	local result = ''
@@ -415,7 +492,8 @@ function run_test(filename, basedir)
 			host	 = PROXY_HOST,
 			port	 = PROXY_PORT,
 			["test-file"] = basedir .. "/t/" .. testname .. ".test",
-			["result-file"] = basedir .. "/r/" .. testname .. ".result"
+			["result-file"] = basedir .. "/r/" .. testname .. ".result",
+			["logdir"] = builddir, -- the .result dir might not be writable
 		})
 	)
 	if USE_POPEN then
@@ -546,7 +624,15 @@ function start_proxy(proxy_name, proxy_options)
 			global_basedir .. 
 			'/t/' ..  proxy_options['proxy-lua-script'] 
 	end
-	-- print_verbose("starting " .. proxy_name)
+	print_verbose("starting " .. proxy_name .. " with " .. options_tostring(proxy_options))
+
+	-- remove the old pid-file if it exists
+	os.remove(proxy_options['pid-file'])
+	-- if we are supposed to listen on a UNIX socket, remove it first, because we don't clean it up on exit!
+	-- TODO: fix the code, instead of hacking around here! Bug#38415
+	if proxy_options['proxy-address'] == '/tmp/mysql-proxy-test.sock' then
+		os.remove(proxy_options['proxy-address'])
+	end
 	-- os.execute("head " .. proxy_options['proxy-lua-script'])
 	assert(os.execute( 'LUA_PATH="' .. INCLUDE_PATH  .. '"  ' ..
 		PROXY_TRACE .. " " .. PROXY_BINPATH .. " " ..
@@ -594,34 +680,61 @@ end
 -- default backend server, and the second one (with default ports)
 -- is pointing at the first proxy
 --
--- @param first_lua_script
+--   client -> proxy -> backend_proxy -> [ mysql-backend ]
+--
+-- usually we use it to mock a server with a lua script (...-mock.lua) and
+-- the script under test in the proxy (...-test.lua)
+--
+-- in case you want to start several backend_proxies, just provide a array
+-- as first param
+--
+-- @param backend_lua_script
 -- @param second_lua_script 
 -- @param use_replication uses a master proxy as backend 
-function chain_proxy (first_lua_script, second_lua_script, use_replication)
-	first_proxy_options = {
-			["proxy-backend-addresses"] = MYSQL_HOST .. ":" .. MYSQL_PORT,
-			["proxy-address"]		   = PROXY_HOST .. ":" .. PROXY_CHAIN_PORT,
-			["admin-address"]		   = PROXY_HOST .. ":" .. ADMIN_CHAIN_PORT,
-			["pid-file"]				= PROXY_CHAIN_PIDFILE,
-			["proxy-lua-script"]		= first_lua_script or DEFAULT_SCRIPT_FILENAME,
-	}
-	-- 
-	-- if replication was not started, then it is started here
-	--
-	if use_replication and (use_replication == true) then
-		if (proxy_list['master'] == nil) then
-			simulate_replication()
-		end
-		first_proxy_options["proxy-backend-addresses"] = PROXY_HOST .. ':' .. PROXY_MASTER_PORT
+function chain_proxy (backend_lua_scripts, second_lua_script, use_replication)
+	local backends = { }
+
+	if type(backend_lua_scripts) == "table" then
+		backends = backend_lua_scripts
+	else
+		backends = { backend_lua_scripts }
 	end
+
+	local backend_addresses = { }
+
+	for i, backend_lua_script in ipairs(backends) do
+		backend_addresses[i] = PROXY_HOST .. ":" .. (PROXY_CHAIN_PORT + i - 1)
+
+		backend_proxy_options = {
+			["proxy-backend-addresses"] = MYSQL_HOST .. ":" .. MYSQL_PORT,
+			["proxy-address"]		   = backend_addresses[i],
+			["admin-address"]		   = PROXY_HOST .. ":" .. (ADMIN_CHAIN_PORT + i - 1),
+			["pid-file"]				= PROXY_CHAIN_PIDFILE .. i,
+			["proxy-lua-script"]		= backend_lua_script or DEFAULT_SCRIPT_FILENAME,
+			["plugin-dir"]			= PROXY_LIBPATH,
+			["basedir"]					= PROXY_TEST_BASEDIR,
+		}
+		-- 
+		-- if replication was not started, then it is started here
+		--
+		if use_replication and (use_replication == true) then
+			if (proxy_list['master'] == nil) then
+				simulate_replication()
+			end
+			backend_proxy_options["proxy-backend-addresses"] = PROXY_HOST .. ':' .. PROXY_MASTER_PORT
+		end
+		start_proxy('backend_proxy' .. i, backend_proxy_options) 
+	end
+
 	second_proxy_options = {
-			["proxy-backend-addresses"] = MYSQL_HOST .. ":" .. PROXY_CHAIN_PORT ,
+			["proxy-backend-addresses"] = backend_addresses ,
 			["proxy-address"]		   	= PROXY_HOST .. ":" .. PROXY_PORT,
 			["admin-address"]		   	= PROXY_HOST .. ":" .. ADMIN_PORT,
 			["pid-file"]				= PROXY_PIDFILE,
 			["proxy-lua-script"]		= second_lua_script or DEFAULT_SCRIPT_FILENAME,
+			["plugin-dir"]			= PROXY_LIBPATH,
+			["basedir"]					= PROXY_TEST_BASEDIR,
 	}
-	start_proxy('first_proxy', first_proxy_options) 
 	start_proxy('second_proxy',second_proxy_options) 
 end
 
@@ -639,11 +752,13 @@ file_empty(DEFAULT_SCRIPT_FILENAME)
 -- otherwise execute all tests we can find
 if #arg then
 	for i, a in ipairs(arg) do
-		local stat = lfs.attributes(a)
+		local stat = assert(lfs.attributes(a))
 
 		if file_exists(a .. '/' .. tests_to_skip_filename) then
 			assert(loadfile(a .. '/' .. tests_to_skip_filename))()
 		end
+
+		-- if it is a directory, execute all of them
 		if stat.mode == "directory" then
 			for file in lfs.dir(a .. "/t/") do
 				local testname = file:match("(.+\.test)$")
@@ -673,7 +788,10 @@ if #arg then
 				end
 			end
 		else 
-			exitcode, skipped = run_test(a)
+			-- otherwise just this one test
+			--
+			-- FIXME: base/ is hard-coded for now
+			exitcode, skipped = run_test(a, "base/")
 			num_skipped = num_skipped + skipped
 		end
 	end
